﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using MindFusion.Charting.Wpf;

namespace PieChart3D
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        private BrushCollection brushesList;
        private Random r = new Random();
        private bool unlock;	

        public Window1()
        {
            InitializeComponent();

            Background = new LinearGradientBrush(Colors.SteelBlue, Colors.White, 0);

            pieChart3D.Series.Clear();

            //Initialize data		
            brushesList = new BrushCollection();
            byte t = 200;
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.DodgerBlue.Color.R, Brushes.DodgerBlue.Color.G, Brushes.DodgerBlue.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.Brown.Color.R, Brushes.Brown.Color.G, Brushes.Brown.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.Orange.Color.R, Brushes.Orange.Color.G, Brushes.Orange.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.Yellow.Color.R, Brushes.Yellow.Color.G, Brushes.Yellow.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.Green.Color.R, Brushes.Green.Color.G, Brushes.Green.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.Azure.Color.R, Brushes.Azure.Color.G, Brushes.Azure.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.LightSteelBlue.Color.R, Brushes.LightSteelBlue.Color.G, Brushes.LightSteelBlue.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.SteelBlue.Color.R, Brushes.SteelBlue.Color.G, Brushes.SteelBlue.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.DarkBlue.Color.R, Brushes.DarkBlue.Color.G, Brushes.DarkBlue.Color.B)));
            brushesList.Add(new SolidColorBrush(Color.FromArgb(t,
                Brushes.DarkViolet.Color.R, Brushes.DarkViolet.Color.G, Brushes.DarkViolet.Color.B)));


            PieSeries serie = new PieSeries();
            serie.Data = new DoubleCollection() { 25, 44, 72, 55, 37 };
            serie.DetachedSlicesList = new DoubleCollection() { 0, 0, 0, 0, 0 };
            serie.OuterLabelType = LabelType.Percents;           
            serie.Fills = brushesList;
            pieChart3D.Series.Add(serie);

            UpdatePieceSelection();
            pieceSelection.SelectedIndex = 0;

            widthSlider.Value = 0.6;
            heightSlider.Value = 0.10;
            donSlider.Value = 1.0 / 3.0;
            cornerSlider.Value = 1.0 / 2.5;
            innerSlider.Value = serie.InnerLabelLocation;
            serie.OuterLabelOffset = 25;
            outerSlider.Value = serie.OuterLabelOffset;
            unlock = true;
        }
        private void UpdatePieceSelection()
        {
            int j = pieceSelection.SelectedIndex;
            pieceSelection.Items.Clear();
            for (int i = 0; i < (pieChart3D.Series[0] as PieSeries).Data.Count; i++)
                pieceSelection.Items.Add("Piece " + i.ToString());

            if (j >= (pieChart3D.Series[0] as PieSeries).Data.Count)
                j = (pieChart3D.Series[0] as PieSeries).Data.Count - 1;
            pieceSelection.SelectedIndex = j;
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            pieChart3D.BeginInit();
            PieSeries serie = pieChart3D.Series[0] as PieSeries;
            double d = r.Next(15, 75);
            serie.Data.Add(d);
            serie.DetachedSlicesList.Add(0.0);
			if (pieChart3D.SliceLayout == SliceLayout.Cascading)
                pieChart3D.ModelHeight = (100.0 * heightSlider.Value) * serie.Data.Count;
            else
                pieChart3D.ModelHeight = 100.0 * heightSlider.Value;
            UpdatePieceSelection();
            pieChart3D.EndInit();
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            pieChart3D.BeginInit();
            PieSeries serie = pieChart3D.Series[0] as PieSeries;
            if (serie.Data.Count > 0)
            {
                int g = serie.Data.Count - 1;
                serie.Data.Remove(serie.Data[g]);
				serie.DetachedSlicesList.Remove(serie.DetachedSlicesList[g]);
            }
			if (pieChart3D.SliceLayout == SliceLayout.Cascading)
                pieChart3D.ModelHeight = (100.0 * heightSlider.Value) * serie.Data.Count;
            else
                pieChart3D.ModelHeight = 100.0 * heightSlider.Value;
            UpdatePieceSelection();
            pieChart3D.EndInit();
        }

        private void Slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            PieSeries serie = pieChart3D.Series[0] as PieSeries;
			if (pieceSelection.SelectedIndex > -1 && pieceSelection.SelectedIndex < serie.DetachedSlicesList.Count)
				serie.DetachedSlicesList[pieceSelection.SelectedIndex] = detachSlider.Value;
        }

        private void cornerSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (unlock)
            {
                if (double.IsNaN(pieChart3D.ModelHeight))
                    pieChart3D.CornerRadius3D = 5 * cornerSlider.Value;
                else
                    pieChart3D.CornerRadius3D = (pieChart3D.ModelHeight / 2) * cornerSlider.Value;
            }
        }

        private void donSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (unlock)
            {
                if (double.IsNaN(pieChart3D.ModelWidth))
                    pieChart3D.DoughnutThickness = 30 * donSlider.Value;
                else
                    pieChart3D.DoughnutThickness = (pieChart3D.ModelWidth / 2) * donSlider.Value;
            }
        }

        private void roundCornerSelector_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!unlock)
                return;

            if (roundCornerSelector.SelectedIndex == 0)
                pieChart3D.RoundCornerType = RoundCornerType.Top;
            else if (roundCornerSelector.SelectedIndex == 1)
                pieChart3D.RoundCornerType = RoundCornerType.Bottom;
            else if (roundCornerSelector.SelectedIndex == 2)
                pieChart3D.RoundCornerType = RoundCornerType.All;
            else if (roundCornerSelector.SelectedIndex == 3)
                pieChart3D.RoundCornerType = RoundCornerType.None;
        }

        private void squireSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (unlock)
                pieChart3D.MeshFaceSize = 5.0 * squireSlider.Value + 0.5;
        }

        private void widthSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (unlock)
            {
                pieChart3D.ModelWidth = 100.0 * widthSlider.Value;
                pieChart3D.ModelDepth = 100.0 * widthSlider.Value;
            }
        }

        private void heightSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (unlock)
            {
                PieSeries serie = pieChart3D.Series[0] as PieSeries;

				if (pieChart3D.SliceLayout == SliceLayout.Cascading)
                    pieChart3D.ModelHeight = (100.0 * heightSlider.Value) * serie.Data.Count;
                else
                    pieChart3D.ModelHeight = 100.0 * heightSlider.Value;
            }
        }

        private void innerLabelSelector_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!unlock)
                return;

            if (innerLabelSelector.SelectedIndex == 0)
                (pieChart3D.Series[0] as PieSeries).InnerLabelType = LabelType.Percents;
            if (innerLabelSelector.SelectedIndex == 1)
                (pieChart3D.Series[0] as PieSeries).InnerLabelType = LabelType.AutoScale;
            if (innerLabelSelector.SelectedIndex == 2)
                (pieChart3D.Series[0] as PieSeries).InnerLabelType = LabelType.Empty;
        }

        private void outerLabelSelector_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!unlock)
                return;

            if (outerLabelSelector.SelectedIndex == 0)
                (pieChart3D.Series[0] as PieSeries).OuterLabelType = LabelType.Percents;
            if (outerLabelSelector.SelectedIndex == 1)
                (pieChart3D.Series[0] as PieSeries).OuterLabelType = LabelType.AutoScale;
            if (outerLabelSelector.SelectedIndex == 2)
                (pieChart3D.Series[0] as PieSeries).OuterLabelType = LabelType.Empty;
        }

        private void innerSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (unlock && (pieChart3D.Series[0] as PieSeries).InnerLabelType != LabelType.Empty)
                (pieChart3D.Series[0] as PieSeries).InnerLabelLocation = innerSlider.Value;
        }

        private void outerSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (unlock && (pieChart3D.Series[0] as PieSeries).OuterLabelType != LabelType.Empty)
                (pieChart3D.Series[0] as PieSeries).OuterLabelOffset = outerSlider.Value;
        }

        private void detachSelector_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!unlock)
                return;

            if (detachSelector.SelectedIndex == 0)
                pieChart3D.DetachOrientation = Orientation.Horizontal;
            else if (detachSelector.SelectedIndex == 1)
                pieChart3D.DetachOrientation = Orientation.Vertical;
        }

        private void pieceLayoutSelector_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!unlock)
                return;
            pieChart3D.BeginInit();
            if (pieceLayoutSelector.SelectedIndex == 0)
            {
                pieChart3D.ModelHeight = 100.0 * heightSlider.Value;
				pieChart3D.SliceLayout = SliceLayout.Flat;
            }
            else if (pieceLayoutSelector.SelectedIndex == 1)
            {
                pieChart3D.ModelHeight = (100.0 * heightSlider.Value) * (pieChart3D.Series[0] as PieSeries).Data.Count;
				pieChart3D.SliceLayout = SliceLayout.Cascading;
            }
            pieChart3D.EndInit();
        }

        private void behavior_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if ((sender as ComboBox).SelectedIndex == 0)
                pieChart3D.Behavior = Behavior3D.ChangePerspective;
            else
                pieChart3D.Behavior = Behavior3D.ChangeValue;
        }

    }
}
